"{ Package: 'jv:libgdbs/tests' }"

"{ NameSpace: Smalltalk }"

TestCase subclass:#GDBDebuggerTestCase
	instanceVariableNames:'debugger'
	classVariableNames:''
	poolDictionaries:''
	category:'GDB-Core-Tests'
!

!GDBDebuggerTestCase class methodsFor:'queries'!

isAbstract
    "Return if this class is an abstract class.
     True is returned here for myself only; false for subclasses.
     Abstract subclasses must redefine again."

    ^ self == GDBDebuggerTestCase.
! !

!GDBDebuggerTestCase methodsFor:'utilities'!

do: block thenWaitFor: announcementClass1
    ^ self with: debugger do: block thenWaitFor: announcementClass1

    "Created: / 27-02-2015 / 11:44:19 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

with: debugger do: block thenWaitFor: announcementClass1
    | announcements |

    announcements := self with: debugger do: block thenWaitForSequence: { announcementClass1 }.
    announcements notNil ifTrue:[ 
        self assert: announcements size == 1.
        ^ announcements last.
    ].
    ^ nil

    "Created: / 01-08-2014 / 12:58:44 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

with: debugger do: block thenWaitFor: announcementClass1 thenWaitFor: announcementClass2
    | announcements |

    announcements := self with: debugger do: block thenWaitForSequence: { announcementClass1 . announcementClass2 }.
    announcements notNil ifTrue:[ 
        self assert: announcements size == 2.
        ^ announcements last.
    ].
    ^ nil

    "Created: / 01-08-2014 / 12:58:59 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

with: debugger do: block thenWaitFor: announcementClass1 thenWaitFor: announcementClass2  thenWaitFor: announcementClass3
    | announcements |

    announcements := self with: debugger do: block thenWaitForSequence: { announcementClass1 . announcementClass2 . announcementClass3 }.
    announcements notNil ifTrue:[ 
        self assert: announcements size == 3.
        ^ announcements last.
    ].
    ^ nil

    "Created: / 01-08-2014 / 12:59:07 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

with: debugger do: block thenWaitForSequence: announcementClasses
    "Blocks the caller until  given announcement (or its subclass)
     is announced until timeout. Returns the annoucement or nil if call
     timeouted."

    | handlers blocker announcements timeout |

    blocker := Semaphore new.

    handlers        := Array new: announcementClasses size.
    announcements   := Array new: announcementClasses size.

    announcementClasses withIndexDo:[ :announcementClass :announcementClassIndex |
        handlers at: announcementClassIndex put: [ :arg | 
            (announcementClassIndex == 1 or:[ (announcements at: announcementClassIndex - 1) notNil ]) ifTrue:[ 
                "Previous announcement arrived, so record announcement and unregister"
                announcements at: announcementClassIndex put: arg.
                debugger announcer unsubscribe: (handlers at: announcementClassIndex).
                handlers at: announcementClassIndex put: nil.
                announcementClassIndex == handlers size ifTrue:[ 
                    blocker signal
                ].
            ].
        ].
        debugger announcer when: announcementClass do: (handlers at: announcementClassIndex).
    ].
    block value.

    timeout := DefaultWaitForTimeout.
    (blocker waitWithTimeoutMs: timeout) isNil ifTrue:[
        announcements := nil.
    ].

    handlers do:[:handler | handler notNil ifTrue:[debugger announcer unsubscribe: handler] ].
    announcements isNil ifTrue:[
        Logger trace: 'Wait for event(s) timed out'.
        TimeoutError newException
            parameter: timeout;
            raise.
    ].
    ^ announcements

    "Created: / 01-08-2014 / 12:57:36 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 27-02-2015 / 11:43:09 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

